@@ -26,25 +26,29 @@
 #define MAX2(x, y) ((x) > (y) ? (x) : (y))
 #define MAX3(x, y, z) ((x) > (y) ? ((x) > (z) ? (x) : (z)) : ((y) > (z) ? (y) : (z)))
 
-/* locate the center of the movement. */
-struct coord alg_locate_center (struct images *imgs, int width, int height)
+/* locate the center and size of the movement. */
+struct coord alg_locate_center_size(struct images *imgs, int width, int height)
 {
 	char *out=imgs->out;
 	int *labels=imgs->labels;
-	int x, y, centc=0;
+	int x, y, centc=0, xdist=0, ydist=0;
 	struct coord cent;
 
 	cent.x=0;
 	cent.y=0;
+	cent.maxx=0;
+	cent.maxy=0;
+	cent.minx=width;
+	cent.miny=height;
 	
 	/* If Labeling enabled - locate center of largest label */
 	if (imgs->labelsize_max) {
 		/* Locate largest label */
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
-				if (*(labels++)==imgs->largest_label) {
-					cent.x+=x;
-					cent.y+=y;
+				if (*(labels++) == imgs->largest_label) {
+					cent.x += x;
+					cent.y += y;
 					centc++;
 				}
 			}
@@ -54,8 +58,8 @@ struct coord alg_locate_center (struct images *imgs, int width, int height)
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
 				if (*(out++)) {
-					cent.x+=x;
-					cent.y+=y;
+					cent.x += x;
+					cent.y += y;
 					centc++;
 				}
 			}
@@ -65,31 +69,27 @@ struct coord alg_locate_center (struct images *imgs, int width, int height)
 		cent.x=cent.x/centc;
 		cent.y=cent.y/centc;
 	}
-	return (cent);
-}
-
-/* draw a box around the movement */
-void alg_locate (struct coord *cent, struct images *imgs, int width, int height, char *new)
-{
-	char *out=imgs->out;
-	int *labels=imgs->labels;
-	int x, y, maxx=0, maxy=0, minx=width, miny=height;
-	int centc=0, xdist=0, ydist=0;
+	
+	/* Now we find the size of the Motion */
 
+	/* First reset pointers back to initial value */	
 	centc=0;
-	/* If Labeling enabled draw box around largest label instead */
+	labels=imgs->labels;
+	out=imgs->out;
+
+	/* If Labeling then we find the area around largest label instead */
 	if (imgs->labelsize_max) {
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
 				if (*(labels++)==imgs->largest_label) {
-					if (x > cent->x)
-						xdist+=x-cent->x;
-					else if (x < cent->x)
-						xdist+=cent->x-x;
-					if (y > cent->y)
-						ydist+=y-cent->y;
-					else if (y < cent->y)
-						ydist+=cent->y-y;
+					if (x > cent.x)
+						xdist += x - cent.x;
+					else if (x < cent.x)
+						xdist += cent.x - x;
+					if (y > cent.y)
+						ydist += y - cent.y;
+					else if (y < cent.y)
+						ydist += cent.y - y;
 					centc++;
 				}
 			}	
@@ -98,62 +98,95 @@ void alg_locate (struct coord *cent, struct images *imgs, int width, int height,
 		for (y=0; y<height; y++) {
 			for (x=0; x<width; x++) {
 				if (*(out++)) {
-					if (x > cent->x)
-						xdist+=x-cent->x;
-					else if (x < cent->x)
-						xdist+=cent->x-x;
-					if (y > cent->y)
-						ydist+=y-cent->y;
-					else if (y < cent->y)
-						ydist+=cent->y-y;
+					if (x > cent.x)
+						xdist += x - cent.x;
+					else if (x < cent.x)
+						xdist += cent.x - x;
+					if (y > cent.y)
+						ydist += y - cent.y;
+					else if (y < cent.y)
+						ydist += cent.y - y;
 					centc++;
 				}
 			}	
 		}
 	}
+	
 	if (centc) {
-		minx=cent->x-xdist/centc*2;
-		maxx=cent->x+xdist/centc*2;
+		cent.minx = cent.x - xdist/centc*2;
+		cent.maxx = cent.x + xdist/centc*2;
 		/* Make the box a litle bigger in y direction to make sure the
-		 * heads fit in */
-		miny=cent->y-ydist/centc*3;
-		maxy=cent->y+ydist/centc*2;
+		   heads fit in so we multiply by 3 instead of 2 which seems to
+		   to work well in practical */
+		cent.miny = cent.y - ydist/centc*3;
+		cent.maxy = cent.y + ydist/centc*2;
 	}
-	if (maxx > width-1)
-		maxx=width-1;
-	else if (maxx < 0)
-		maxx=0;
-	if (maxy > height-1)
-		maxy=height-1;
-	else if (maxy < 0)
-		maxy=0;
-	if (minx > width-1)
-		minx=width-1;
-	else if (minx < 0)
-		minx=0;
-	if (miny > height-1)
-		miny=height-1;
-	else if (miny < 0)
-		miny=0;
-	cent->width=maxx-minx;
-	cent->height=maxy-miny;
+	if (cent.maxx > width - 1)
+		cent.maxx = width - 1;
+	else if (cent.maxx < 0)
+		cent.maxx = 0;
+	if (cent.maxy > height - 1)
+		cent.maxy = height - 1;
+	else if (cent.maxy < 0)
+		cent.maxy = 0;
+	if (cent.minx > width - 1)
+		cent.minx = width - 1;
+	else if (cent.minx < 0)
+		cent.minx = 0;
+	if (cent.miny > height - 1)
+		cent.miny = height - 1;
+	else if (cent.miny < 0)
+		cent.miny = 0;
+	
+	cent.width = cent.maxx - cent.minx;
+	cent.height = cent.maxy - cent.miny;
+	
+	/* We want to center Y coordinate to be the center of the action.
+	   The head of a person is important so we correct the cent.y coordinate
+	   to match the correction to include a persons head that we just did above */
+	cent.y = (cent.miny + cent.maxy)/2;
+	
+	return (cent);
+}
+
+
+/* draw a box around the movement */
+void alg_draw_location(struct coord *cent, struct images *imgs, int width, int height, char *new, int mode)
+{
+	char *out=imgs->out;
+	int x, y;
+
 	out=imgs->out;
 	
 	/* Draw a box around the movement */
-	for (x=minx; x<=maxx; x++) {
-		new[x+width*miny]=~new[x+width*miny];
-		new[x+width*maxy]=~new[x+width*maxy];
-		out[x+width*miny]=~out[x+width*miny];
-		out[x+width*maxy]=~out[x+width*maxy];
+	if (mode == LOCATE_BOTH){ /* both normal and motion image gets a box */
+		for (x=cent->minx; x<=cent->maxx; x++) {
+			new[x+width*cent->miny]=~new[x+width*cent->miny];
+			new[x+width*cent->maxy]=~new[x+width*cent->maxy];
+			out[x+width*cent->miny]=~out[x+width*cent->miny];
+			out[x+width*cent->maxy]=~out[x+width*cent->maxy];
+		}
+		for (y=cent->miny; y<=cent->maxy; y++) {
+			new[cent->minx+y*width]=~new[cent->minx+y*width];
+			new[cent->maxx+y*width]=~new[cent->maxx+y*width];
+			out[cent->minx+y*width]=~out[cent->minx+y*width];
+			out[cent->maxx+y*width]=~out[cent->maxx+y*width];
+		}
 	}
-	for (y=miny; y<=maxy; y++) {
-		new[minx+y*width]=~new[minx+y*width];
-		new[maxx+y*width]=~new[maxx+y*width];
-		out[minx+y*width]=~out[minx+y*width];
-		out[maxx+y*width]=~out[maxx+y*width];
+	else{ /* normal image only (e.g. preview shot) */
+		for (x=cent->minx; x<=cent->maxx; x++) {
+			new[x+width*cent->miny]=~new[x+width*cent->miny];
+			new[x+width*cent->maxy]=~new[x+width*cent->maxy];
+		}
+		for (y=cent->miny; y<=cent->maxy; y++) {
+			new[cent->minx+y*width]=~new[cent->minx+y*width];
+			new[cent->maxx+y*width]=~new[cent->maxx+y*width];
+		}
 	}
 }
 
+
+
 #define NORM               100
 #define ABS(x)             ((x)<0 ? -(x) : (x))
 #define DIFF(x, y)         (ABS((x)-(y)))
@@ -170,28 +203,27 @@ void alg_noise_tune (struct context *cnt, unsigned char *new)
 	unsigned char *smartmask=imgs->smartmask_final;
 
 	i=imgs->motionsize;
+	
 	for (; i>0; i--) {
 		diff=*ref-*new;
+		
 		if (mask)
 			diff=((diff**mask++)/255);
-		if (diff) {
-			if (*smartmask){
-				sum+=ABS(diff);
-				count++;
-			}
+
+		if (*smartmask){
+			sum+=ABS(diff)+1;
+			count++;
 		}
+
 		ref++;
 		new++;
 		smartmask++;
 	}
-	sum/=count/4;
+	
+	if (count>4) //avoid divide by zero
+		sum/=count/4;
+		
 	cnt->noise=4+(cnt->noise+sum)/2;
-
-	/* If the always_changes option is enabled we assume the user also
-	 * wants to see the changes when the noise level is tuned
-	 */
-	if (cnt->conf.always_diff)
-		printf("New noise level: %d\n", cnt->noise);
 }
 
 void alg_threshold_tune (struct context *cnt, int diffs, int motion)
@@ -202,7 +234,6 @@ void alg_threshold_tune (struct context *cnt, int diffs, int motion)
 	if (!diffs)
 		return;
 
-	printf("%d\n", diffs);
 	if (motion)
 		diffs=cnt->threshold/4;
 
@@ -224,7 +255,6 @@ void alg_threshold_tune (struct context *cnt, int diffs, int motion)
 		sum=top*2;
 	if (sum < cnt->conf.max_changes)
 			cnt->threshold=(cnt->threshold+sum)/2;
-	printf("New threshold: %d\n", cnt->threshold);
 }
 
 /*
@@ -330,7 +360,7 @@ int alg_labeling (struct context *cnt)
 				continue;
 			labelsize=iflood(ix, iy, width, height, out, labels, current_label);
 			if( labelsize > 0 ) {
-				//printf( "Label: %i (%i) Size: %i (%i,%i)\n", current_label, imgs->total_labels, labelsize, ix, iy );//jw
+				//printf( "Label: %i (%i) Size: %i (%i,%i)\n", current_label, imgs->total_labels, labelsize, ix, iy );
 				if( imgs->labelsize_max < labelsize ){
 					imgs->labelsize_max=labelsize;
 					imgs->largest_label=current_label;
@@ -567,37 +597,45 @@ static int erode5(char *img, int width, int height, void *buffer, unsigned char
 /* 
  * Despeckling routine to remove noisy detections.
  */
-int alg_despeckle(struct context *cnt)
+int alg_despeckle(struct context *cnt, int olddiffs)
 {
 	int diffs = 0;
 	char *out = cnt->imgs.out;
 	int width = cnt->imgs.width;
 	int height= cnt->imgs.height;
-	int i, len = strlen(cnt->conf.despeckle);
+	int done = 0, i, len = strlen(cnt->conf.despeckle);
 	unsigned char *common_buffer = cnt->imgs.common_buffer;
 
 	for (i = 0; i < len; i++) {
 		switch (cnt->conf.despeckle[i]) {
 		case 'E':
 			if ((diffs = erode9(out, width, height, common_buffer, 0)) == 0) i=len;
+			done=1;
 			break;
 		case 'e':
 			if ((diffs = erode5(out, width, height, common_buffer, 0)) == 0) i=len;
+			done=1;
 			break;
 		case 'D':
 			diffs = dilate9(out, width, height, common_buffer);
+			done=1;
 			break;
 		case 'd':
 			diffs = dilate5(out, width, height, common_buffer);
+			done=1;
 			break;
 		/* no further despeckle after labeling! */
 		case 'l':
 			diffs = alg_labeling(cnt);
 			i=len;
+			done=1;
 			break;
 		}
 	}
-	return diffs;
+	if (done)
+		return diffs;
+	else
+		return olddiffs;
 }
 
 /* Generate actual smartmask. Calculate sensitivity based on motion */
@@ -994,8 +1032,8 @@ int alg_switchfilter (struct context *cnt, int diffs, struct coord *cent, char *
 			char tmp[80];
 			sprintf(tmp, "%d %d", lines, vertlines);
 			draw_text(newimg, cnt->imgs.width-10, 20, cnt->imgs.height, cnt->imgs.width, tmp);
-			return diffs;
 		}
+		return diffs;
 	}
 	return 0;
 }
