@@ -19,20 +19,20 @@
 	more parameters may be added later.
  */
 struct config {
+	int setup_mode;
 	int width;
 	int height;
 	int quality;
 	int rotate_deg;
 	int max_changes;
 	int threshold_tune;
-	int always_diff;
 	char *output_normal;
 	int motion_img;
 	int output_all;
 	int gap;
 	int maxmpegtime;
 	int snapshot_interval;
-	int locate;
+	char *locate;
 	int input;
 	int norm;
 	int frame_limit;
@@ -68,7 +68,10 @@ struct config {
 	int webcam_limit;
 	int control_port;
 	int control_localhost;
+	int control_html_output;
+	char *control_authentication;
 	int frequency;
+	int tuner_number;
 	int timelapse;
 	char *timelapse_mode; 
 #ifdef __freebsd__
@@ -76,14 +79,13 @@ struct config {
 #endif
 	char *video_device;
 	char *vidpipe;
-	char *mail_address;
-	char *sms_nr;
 	char *filepath;
 	char *jpegpath;
 	char *mpegpath;
 	char *snappath;
 	char *timepath;
-	char *externcommand;
+	char *on_event_start;
+	char *on_event_end;
 	char *mask_file;
 	int smart_mask_speed;
 	int sql_log_image;
@@ -94,9 +96,10 @@ struct config {
 	char *mysql_host;
 	char *mysql_user;
 	char *mysql_password;
-	char *onsavecommand;
-	char *onmpegcommand;
-	char *onffmpegclosecommand;
+	char *on_picture_save;
+	char *on_motion_detected;
+	char *on_movie_start;
+	char *on_movie_end;
 	char *motionvidpipe;
 	char *netcam_url;
 	char *netcam_userpass;
@@ -110,7 +113,7 @@ struct config {
 	char *text_right;
 	char *despeckle;
 	int minimum_motion_frames;
-	int debug_parameter;
+	// int debug_parameter;
 	int argc;
 	char **argv;
 };
@@ -120,7 +123,7 @@ extern struct config conf;
 /** 
  * typedef for a param copy function. 
  */
-typedef struct context ** (* conf_copy_func)(struct context **, char *, int) ;
+typedef struct context ** (* conf_copy_func)(struct context **, char *, int);
 typedef char *(* conf_print_func)(struct context **, char **, int, int);
 
 /**
