@@ -1,3 +1,13 @@
+/*
+ *	event.h
+ *
+ *	Include file for event.c
+ *
+ *	Copyright Jeroen Vreeken, 2002
+ *	This software is distributed under the GNU Public License Version 2
+ *	see also the file 'COPYING'.
+ *
+ */
 #ifndef _INCLUDE_EVENT_H_
 #define _INCLUDE_EVENT_H_
 
@@ -22,5 +32,6 @@
 typedef void(* event_handler)(int, struct context *, char *, void *, struct tm *);
 
 void event(int, struct context *, char *, void *, struct tm *);
+char * imageext(struct context *);
 
 #endif /* _INCLUDE_EVENT_H_ */
