@@ -142,9 +142,10 @@ void rotate_init(struct context *cnt) {
 	 * we have a value that is safe from changes caused by motion-control.
 	 */
 	if((cnt->conf.rotate_deg % 90) > 0) {
+		const char msg[]="Config option \"rotate\" is invalid: ";
 		cnt->rotate_data.degrees = 0; /* rotate_deg is invalid, so ignore it */
-		syslog(LOG_ERR, "Config option \"rotate\" is invalid: %d",
-			cnt->conf.rotate_deg);
+		printf("[%d] %s%d\n", cnt->threadnr, msg, cnt->conf.rotate_deg);
+		syslog(LOG_ERR, "[%d] %s%d", cnt->threadnr, msg, cnt->conf.rotate_deg);
 		return;
 	}
 	else
@@ -197,8 +198,10 @@ void rotate_init(struct context *cnt) {
 		break;
 	default:
 		cnt->rotate_data.degrees = 0;
-		syslog(LOG_ERR, "Unsupported palette (%d), rotation is disabled.",
-			cnt->imgs.type);
+		printf("[%d] Unsupported palette (%d), rotation is disabled.\n",
+			cnt->threadnr, cnt->imgs.type);
+		syslog(LOG_ERR, "[%d] Unsupported palette (%d), rotation is disabled.",
+			cnt->threadnr, cnt->imgs.type);
 		return;
 	}
 
