@@ -42,7 +42,7 @@ static int iomojo_center(struct context *cnt, int xoff, int yoff);
 static int iomojo_move(struct context *cnt, int dev, int pipe, int mpipe, struct coord *cent, struct images *imgs);
 static int lqos_center(struct context *cnt, int dev, int xoff, int yoff);
 static int lqos_move(struct context *cnt, int dev, int pipe, int mpipe, 
-    struct coord *cent, struct images *imgs, int manual);
+                     struct coord *cent, struct images *imgs, int manual);
 
 /* Add a call to your functions here: */
 int track_center(struct context *cnt, int dev, int manual, int xoff, int yoff)
@@ -55,8 +55,11 @@ int track_center(struct context *cnt, int dev, int manual, int xoff, int yoff)
 		return lqos_center(cnt, dev, xoff, yoff);
 	else if (cnt->track.type == TRACK_TYPE_IOMOJO)
 		return iomojo_center(cnt, xoff, yoff);
+	else if (cnt->track.type == TRACK_TYPE_GENERIC)
+		return 10; // FIX ME. I chose to return something reasonable.
 
-	syslog(LOG_ERR, "track_move: internal error, %d is not a known track-type", cnt->track.type);
+	printf("[%d] track_move: internal error, %d is not a known track-type\n", cnt->threadnr, cnt->track.type);
+	syslog(LOG_ERR, "[%d] track_move: internal error, %d is not a known track-type", cnt->threadnr, cnt->track.type);
 
 	return 0;
 }
@@ -72,8 +75,11 @@ int track_move(struct context *cnt, int dev, int pipe, int mpipe, struct coord *
 		return lqos_move(cnt, dev, pipe, mpipe, cent, imgs, manual);
 	else if (cnt->track.type == TRACK_TYPE_IOMOJO)
 		return iomojo_move(cnt, dev, pipe, mpipe, cent, imgs);
+	else if (cnt->track.type == TRACK_TYPE_GENERIC)
+		return 0; // FIX ME. I chose to return something reasonable.
 
-	syslog(LOG_ERR, "track_move: internal error, %d is not a known track-type", cnt->track.type);
+	printf("[%d] track_move: internal error, %d is not a known track-type\n", cnt->threadnr, cnt->track.type);
+	syslog(LOG_ERR, "[%d] track_move: internal error, %d is not a known track-type", cnt->threadnr, cnt->track.type);
 
 	return 0;
 }
@@ -99,7 +105,8 @@ static int stepper_command(struct context *cnt, int motor, int command, int n)
 
 	while (read(cnt->track.dev, buffer, 1)!=1 && time(NULL) < timeout+1);
 	if (time(NULL) >= timeout+2) {
-		syslog(LOG_ERR, "Status byte timeout!");
+		printf("[%d] Status byte timeout!\n", cnt->threadnr);
+		syslog(LOG_ERR, "[%d] Status byte timeout!", cnt->threadnr);
 		return 0;
 	}
 	return buffer[0];
@@ -118,7 +125,9 @@ static int stepper_center(struct context *cnt, int xoff, int yoff)
 
 	if (cnt->track.dev<0) {
 		if ((cnt->track.dev=open(cnt->track.port, O_RDWR | O_NOCTTY)) < 0) {
-			syslog(LOG_ERR, "Unable to open serial device %s: %m", cnt->track.port);
+			const char msg[]="Unable to open serial device ";
+			printf("[%d] %s%s: %m\n", cnt->threadnr, msg, cnt->track.port);
+			syslog(LOG_ERR, "[%d] %s%s: %m", cnt->threadnr, msg, cnt->track.port);
 			exit(1);
 		}
 		bzero (&adtio, sizeof(adtio));
@@ -130,7 +139,9 @@ static int stepper_center(struct context *cnt, int xoff, int yoff)
 		adtio.c_cc[VMIN]=0;	/* blocking read until 1 char */
 		tcflush (cnt->track.dev, TCIFLUSH);
 		if (tcsetattr(cnt->track.dev, TCSANOW, &adtio) < 0) {
-			syslog(LOG_ERR, "Unable to initialize serial device %s: %m", cnt->track.port);
+			const char msg[]="Unable to initialize serial device ";
+			printf("[%d] %s%s: %m\n", cnt->threadnr, msg, cnt->track.port);
+			syslog(LOG_ERR, "[%d] %s%s: %m", cnt->threadnr, msg, cnt->track.port);
 			exit(1);
 		}
 	}
@@ -186,7 +197,9 @@ static char iomojo_command(struct context *cnt, char *command, int len, int ret)
 	if (ret) {
 		while (read(cnt->track.dev, buffer, 1)!=1 && time(NULL) < timeout+2);
 		if (time(NULL) >= timeout+2) {
-			syslog(LOG_ERR, "Return byte timeout!");
+			const char msg[]="Return byte timeout!";
+			printf("[%d] %s\n", cnt->threadnr, msg);
+			syslog(LOG_ERR, "[%d] %s", cnt->threadnr, msg);
 			return 0;
 		}
 	}
@@ -201,8 +214,11 @@ static void iomojo_setspeed(struct context *cnt, int speed)
 	command[1]=cnt->track.iomojo_id;
 	command[2]=speed;
 	
-	if (iomojo_command(cnt, command, 3, 1)!=IOMOJO_SETSPEED_RET)
-		syslog(LOG_ERR, "Unable to set camera speed");
+	if (iomojo_command(cnt, command, 3, 1)!=IOMOJO_SETSPEED_RET){
+		const char msg[]="Unable to set camera speed";
+		printf("[%d] %s\n", cnt->threadnr, msg);
+		syslog(LOG_ERR, "[%d] %s", cnt->threadnr, msg);
+	}
 }
 
 static void iomojo_movehome(struct context *cnt)
@@ -222,7 +238,9 @@ static int iomojo_center(struct context *cnt, int xoff, int yoff)
 
 	if (cnt->track.dev<0) {
 		if ((cnt->track.dev=open(cnt->track.port, O_RDWR | O_NOCTTY)) < 0) {
-			syslog(LOG_ERR, "Unable to open serial device %s: %m", cnt->track.port);
+			const char msg[]="Unable to open serial device ";
+			printf("[%d] %s%s: %m\n", cnt->threadnr, msg, cnt->track.port);
+			syslog(LOG_ERR, "[%d] %s%s: %m", cnt->threadnr, msg, cnt->track.port);
 			return 0;
 		}
 		bzero (&adtio, sizeof(adtio));
@@ -234,7 +252,9 @@ static int iomojo_center(struct context *cnt, int xoff, int yoff)
 		adtio.c_cc[VMIN]=0;	/* blocking read until 1 char */
 		tcflush (cnt->track.dev, TCIFLUSH);
 		if (tcsetattr(cnt->track.dev, TCSANOW, &adtio) < 0) {
-			syslog(LOG_ERR, "Unable to initialize serial device %s: %m", cnt->track.port);
+			const char msg[]="Unable to initialize serial device ";
+			printf("[%d] %s%s: %m\n", cnt->threadnr, msg, cnt->track.port);
+			syslog(LOG_ERR, "[%d] %s%s: %m", cnt->threadnr, msg, cnt->track.port);
 			return 0;
 		}
 	}
@@ -329,12 +349,16 @@ static int lqos_center(struct context *cnt, int dev, int xoff, int yoff)
 
 	if (cnt->track.dev==-1) {
 		if (ioctl(dev, VIDIOCPWCMPTRESET, &reset) == -1) {
-			syslog(LOG_ERR, "Failed to reset camera to starting position! Reason: %m");
+			const char msg[]="Failed to reset camera to starting position! Reason: ";
+			printf("[%d] %s%m\n", cnt->threadnr, msg);
+			syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 			return 0;
 		}
 		sleep(6);
 		if (ioctl(dev, VIDIOCPWCMPTGRANGE, &pmr) == -1) {
-			syslog(LOG_ERR, "failed VIDIOCPWCMPTGRANGE %m");
+			const char msg[]="failed VIDIOCPWCMPTGRANGE ";
+			printf("[%d] %s%m\n", cnt->threadnr, msg);
+			syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 			return 0;
 		}
 		cnt->track.dev=dev;
@@ -345,7 +369,9 @@ static int lqos_center(struct context *cnt, int dev, int xoff, int yoff)
 		cnt->track.tiltmax=pmr.tilt_max;
 	}
 	if (ioctl(dev, VIDIOCPWCMPTGANGLE, &pma) == -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCPWCMPTGANGLE: %m");
+		const char msg[]="ioctl VIDIOCPWCMPTGANGLE: ";
+		printf("[%d] %s%m\n", cnt->threadnr, msg);
+		syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 	}
 	pma.absolute=1;
 	if (xoff*100 < cnt->track.panmax && xoff*100 > cnt->track.panmin)
@@ -353,7 +379,9 @@ static int lqos_center(struct context *cnt, int dev, int xoff, int yoff)
 	if (yoff*100 < cnt->track.tiltmax && yoff*100 > cnt->track.tiltmin)
 		pma.tilt = yoff*100;
 	if (ioctl(dev, VIDIOCPWCMPTSANGLE, &pma) == -1) {
-		syslog(LOG_ERR, "Failed to pan/tilt camera! Reason: %m");
+		const char msg[]="Failed to pan/tilt camera! Reason: ";
+		printf("[%d] %s%m\n", cnt->threadnr, msg);
+		syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 		return 0;
 	}
 
@@ -386,7 +414,9 @@ static int lqos_move(struct context *cnt, int dev, int pipe, int mpipe,
 	/* If we never checked for the min/max values for pan/tilt we do it now */
 	if (cnt->track.minmaxfound==0) {
 		if (ioctl(dev, VIDIOCPWCMPTGRANGE, &pmr) == -1) {
-			syslog(LOG_ERR, "failed VIDIOCPWCMPTGRANGE %m");
+			const char msg[]="failed VIDIOCPWCMPTGRANGE ";
+			printf("[%d] %s%m\n", cnt->threadnr, msg);
+			syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 			return 0;
 		}
 		cnt->track.minmaxfound=1;
@@ -398,7 +428,9 @@ static int lqos_move(struct context *cnt, int dev, int pipe, int mpipe,
 
 	/* Get current camera position */
 	if (ioctl(dev, VIDIOCPWCMPTGANGLE, &pma) == -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCPWCMPTGANGLE: %m");
+		const char msg[]="ioctl VIDIOCPWCMPTGANGLE: ";
+		printf("[%d] %s%m\n", cnt->threadnr, msg);
+		syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 	}
 
 	/* Check current position of camera and see if we need to adjust
@@ -416,9 +448,10 @@ static int lqos_move(struct context *cnt, int dev, int pipe, int mpipe,
 	pma.absolute=0;
 	pma.pan = move_x_degrees;
 	pma.tilt = move_y_degrees;
-	if (ioctl(dev, VIDIOCPWCMPTSANGLE, &pma) == -1)
-	{
-		syslog(LOG_ERR, "Failed to pan/tilt camera! Reason: %m");
+	if (ioctl(dev, VIDIOCPWCMPTSANGLE, &pma) == -1) {
+		const char msg[]="Failed to pan/tilt camera! Reason: ";
+		printf("[%d] %s%m\n", cnt->threadnr, msg);
+		syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 		return 0;
 	}
 
