@@ -24,22 +24,25 @@
 #include <sys/utsname.h>
 #include <dirent.h>
 
+#define MAX2(x, y) ((x) > (y) ? (x) : (y))
+#define MIN2(x, y) ((x) < (y) ? (x) : (y))
+
 static void v4l_picture_controls(struct video_dev *viddev, struct context *cnt)
 {
 	int dev=viddev->fd;
 	unsigned char *image = cnt->imgs.new;
 	struct video_picture vid_pic;
 	int make_change = 0;
-	int i, j = 0, avg = 0, offset = 0;
 
 	if (cnt->conf.contrast && cnt->conf.contrast != viddev->contrast) {
 
 		if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
-			syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+			printf("[%d] ioctl (VIDIOCGPICT): %m\n", cnt->threadnr);
+			syslog(LOG_ERR, "[%d] ioctl (VIDIOCGPICT): %m", cnt->threadnr);
 		}
 
 		make_change = 1;
-		vid_pic.contrast = cnt->conf.contrast * 255;
+		vid_pic.contrast = cnt->conf.contrast * 256;
 		viddev->contrast = cnt->conf.contrast;
 	}
 
@@ -47,12 +50,13 @@ static void v4l_picture_controls(struct video_dev *viddev, struct context *cnt)
 
 		if (!make_change) {
 			if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
-				syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+				printf("[%d] ioctl (VIDIOCGPICT): %m\n", cnt->threadnr);
+				syslog(LOG_ERR, "{%d] ioctl (VIDIOCGPICT): %m", cnt->threadnr);
 			}
 		}
 
 		make_change = 1;
-		vid_pic.colour = cnt->conf.saturation * 255;
+		vid_pic.colour = cnt->conf.saturation * 256;
 		viddev->saturation = cnt->conf.saturation;
 	}
 
@@ -60,39 +64,60 @@ static void v4l_picture_controls(struct video_dev *viddev, struct context *cnt)
 
 		if (!make_change) {
 			if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
-				syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+				printf("[%d] ioctl (VIDIOCGPICT): %m\n", cnt->threadnr);
+				syslog(LOG_ERR, "[%d] ioctl (VIDIOCGPICT): %m", cnt->threadnr);
 			}
 		}
 
 		make_change = 1;
-		vid_pic.hue = cnt->conf.hue * 255;
+		vid_pic.hue = cnt->conf.hue * 256;
 		viddev->hue = cnt->conf.hue;
 	}
 	
 	if (cnt->conf.autobright) {
-		for (i = 0; i < cnt->imgs.width * cnt->imgs.height; i += 100) {
+		
+		int brightness_window_high;
+		int brightness_window_low;
+		int brightness_target;
+		int i, j = 0, avg = 0, step = 0;
+		
+		if (cnt->conf.brightness)
+			brightness_target = cnt->conf.brightness;
+		else
+			brightness_target = 128;
+		
+		brightness_window_high = MIN2(brightness_target + 10, 255);
+		brightness_window_low = MAX2(brightness_target - 10, 1);
+		
+		for (i = 0; i < cnt->imgs.motionsize; i += 101) {
 			avg += *image++;
 			j++;
 		}
 		avg = avg / j;
-
-		if (avg > 140 || avg < 64) {
+		
+		if (avg > brightness_window_high || avg < brightness_window_low) {
+			/* If we already read the VIDIOGPICT - we should not do it again */
 			if (!make_change) {
 				if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
-					syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+					printf("[%d] ioctl (VIDIOCGPICT): %m\n", cnt->threadnr);
+					syslog(LOG_ERR, "[%d] ioctl (VIDIOCGPICT): %m", cnt->threadnr);
 				}
 			}
-			if (avg > 140) {
-				offset=avg-140;
-				if (vid_pic.brightness > 100*offset) {
-					vid_pic.brightness-=100*offset;
+			/* average is above window - turn down brigtness - go for the target */
+			if (avg > brightness_window_high) {
+				step = MIN2((avg - brightness_target)/5+1, viddev->brightness - 1);
+				if (viddev->brightness > 1 ) {
+					viddev->brightness -= step;
+					vid_pic.brightness = viddev->brightness * 256;
 					make_change = 1;
 				}
 			}
-			if (avg < 64) {
-				offset=64-avg;
-				if (vid_pic.brightness < 65535-100*offset) {
-					vid_pic.brightness+=100*offset;
+			/* average is below window - turn up brigtness - go for the target */
+			if (avg < brightness_window_low) {
+				step = MIN2((brightness_target - avg)/5+1, 255 - viddev->brightness);
+				if (viddev->brightness < 255 ) {
+					viddev->brightness += step;
+					vid_pic.brightness = viddev->brightness * 256;
 					make_change = 1;
 				}
 			}
@@ -101,19 +126,21 @@ static void v4l_picture_controls(struct video_dev *viddev, struct context *cnt)
 		if (cnt->conf.brightness && cnt->conf.brightness != viddev->brightness) {
 			if (!make_change) {
 				if (ioctl(dev, VIDIOCGPICT, &vid_pic)==-1) {
-					syslog(LOG_ERR, "ioctl (VIDIOCGPICT): %m");
+					printf("[%d] ioctl (VIDIOCGPICT): %m\n", cnt->threadnr);
+					syslog(LOG_ERR, "[%d] ioctl (VIDIOCGPICT): %m", cnt->threadnr);
 				}
 			}
 	
 			make_change = 1;
-			vid_pic.brightness = cnt->conf.brightness * 255;
+			vid_pic.brightness = cnt->conf.brightness * 256;
 			viddev->brightness = cnt->conf.brightness;
 		}
 	}
 
 	if (make_change) {
 		if (ioctl(dev, VIDIOCSPICT, &vid_pic)==-1) {
-			syslog(LOG_ERR, "ioctl (VIDIOCSPICT): %m");
+			printf("[%d] ioctl (VIDIOCSPICT): %m\n", cnt->threadnr);
+			syslog(LOG_ERR, "[%d] ioctl (VIDIOCSPICT): %m", cnt->threadnr);
 		}
 	}
 }
@@ -185,7 +212,7 @@ void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int h
 			v++;
 		}
 
-		if ((loop & 1) == 1)
+		if ((loop & 1) == 0)
 		{
 			u-=width/2;
 			v-=width/2;
@@ -198,7 +225,8 @@ void rgb24toyuv420p(unsigned char *map, unsigned char *cap_map, int width, int h
 */
 
 
-static char *v4l_start (struct video_dev *viddev, int width, int height, int input, int norm, unsigned long freq)
+static char *v4l_start (struct video_dev *viddev, int width, int height, int input, int norm,
+                        unsigned long freq, int tuner_number, struct context *cnt)
 {
 	int dev=viddev->fd;
 	struct video_capability vid_caps;
@@ -210,7 +238,8 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	char *map;
 
 	if (ioctl (dev, VIDIOCGCAP, &vid_caps) == -1) {
-		syslog(LOG_ERR, "ioctl (VIDIOCGCAP): %m");
+		printf("[%d] ioctl (VIDIOCGCAP): %m\n", cnt->threadnr);
+		syslog(LOG_ERR, "[%d] ioctl (VIDIOCGCAP): %m", cnt->threadnr);
 		return (NULL);
 	}
 	if (vid_caps.type & VID_TYPE_MONOCHROME) viddev->v4l_fmt=VIDEO_PALETTE_GREY;
@@ -218,19 +247,23 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 		memset(&vid_chnl, 0, sizeof(struct video_channel));
 		vid_chnl.channel = input;
 		if (ioctl (dev, VIDIOCGCHAN, &vid_chnl) == -1) {
-			syslog(LOG_ERR, "ioctl (VIDIOCGCHAN): %m");
+			printf("[%d] ioctl (VIDIOCGCHAN): %m\n", cnt->threadnr);
+			syslog(LOG_ERR, "[%d] ioctl (VIDIOCGCHAN): %m", cnt->threadnr);
 		} else {
 			vid_chnl.channel = input;
 			vid_chnl.norm    = norm;
 			if (ioctl (dev, VIDIOCSCHAN, &vid_chnl) == -1) {
-				syslog(LOG_ERR, "ioctl (VIDIOCSCHAN): %m");
+				printf("[%d] ioctl (VIDIOCSCHAN): %m\n", cnt->threadnr);
+				syslog(LOG_ERR, "[%d] ioctl (VIDIOCSCHAN): %m", cnt->threadnr);
 				return (NULL);
 			}
 		}
 	}
 	if (freq) {
+		vid_tuner.tuner = tuner_number;
 		if (ioctl (dev, VIDIOCGTUNER, &vid_tuner)==-1) {
-			syslog(LOG_ERR, "ioctl (VIDIOCGTUNER): %m");
+			printf("[%d] ioctl (VIDIOCGTUNER): %m\n", cnt->threadnr);
+			syslog(LOG_ERR, "[%d] ioctl (VIDIOCGTUNER): %m", cnt->threadnr);
 		} else {
 			if (vid_tuner.flags & VIDEO_TUNER_LOW) {
 				freq=freq*16; /* steps of 1/16 KHz */
@@ -238,28 +271,37 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 				freq=(freq*10)/625;
 			}
 			if (ioctl(dev, VIDIOCSFREQ, &freq)==-1) {
-				syslog(LOG_ERR, "ioctl (VIDIOCSFREQ): %m");
+				printf("[%d] ioctl (VIDIOCSFREQ): %m\n", cnt->threadnr);
+				syslog(LOG_ERR, "[%d] ioctl (VIDIOCSFREQ): %m", cnt->threadnr);
 				return (NULL);
 			}
-			printf("Frequency set\n");
+			if (cnt->conf.setup_mode)
+				printf("[%d] Frequency set\n", cnt->threadnr);
 		}
 	}
 	if (ioctl (dev, VIDIOCGMBUF, &vid_buf) == -1) {
-		syslog(LOG_WARNING, "no mmap falling back on read");
+		const char msg[]="No mmap falling back on read";
+		const char msg1[]="V4L capturing using read is deprecated!";
+		const char msg2[]="Motion now only supports mmap.";
+		printf("[%d] %s\n", cnt->threadnr, msg);
+		syslog(LOG_WARNING, "[%d] %s", cnt->threadnr, msg);
 		if (ioctl (dev, VIDIOCGWIN, &vid_win)== -1) {
-			syslog(LOG_ERR, "ioctl VIDIOCGWIN: %m");
+			printf("[%d] ioctl VIDIOCGWIN: %m\n", cnt->threadnr);
+			syslog(LOG_ERR, "[%d] ioctl VIDIOCGWIN: %m", cnt->threadnr);
 			return (NULL);
 		}
 		vid_win.width=width;
 		vid_win.height=height;
 		vid_win.clipcount=0;
 		if (ioctl (dev, VIDIOCSWIN, &vid_win)== -1) {
-			syslog(LOG_ERR, "ioctl VIDIOCSWIN: %m");
+			printf("[%d] ioctl VIDIOCSWIN: %m\n",cnt->threadnr);
+			syslog(LOG_ERR, "[%d] ioctl VIDIOCSWIN: %m", cnt->threadnr);
 			return (NULL);
 		}
-		syslog(LOG_WARNING, "V4L capturing using read is deprecated!!!!");
-		syslog(LOG_WARNING, "Send an email to pe1rxq@amsat.org if your device does not support mmap");
-		syslog(LOG_WARNING, "Please also give the kernel and driver versions");
+		syslog(LOG_WARNING, "[%d] %s", cnt->threadnr, msg1);
+		syslog(LOG_WARNING, "[%d] %s", cnt->threadnr, msg2);
+		printf("[%d] %s\n", cnt->threadnr, msg1);
+		printf("[%d] %s\n", cnt->threadnr, msg2);
 		exit(1);
 		viddev->v4l_read_img=1;
 		map=mymalloc(width*height*3);
@@ -268,6 +310,7 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 		viddev->v4l_buffers[0]=map;
 	}else{
 		map=mmap(0, vid_buf.size, PROT_READ|PROT_WRITE, MAP_SHARED, dev, 0);
+		viddev->size_map=vid_buf.size;
 		if (vid_buf.frames>1) {
 			viddev->v4l_maxbuffer=2;
 			viddev->v4l_buffers[0]=map;
@@ -286,17 +329,22 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 		vid_mmap.width=width;
 		vid_mmap.height=height;
 		if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-			syslog(LOG_DEBUG, "Failed with YUV420P, trying YUV422 palette\n");
+			const char msg[]="Failed with YUV420P, trying YUV422 palette";
+			printf("[%d] %s\n", cnt->threadnr, msg);
+			syslog(LOG_DEBUG, "[%d] %s", cnt->threadnr, msg);
 			viddev->v4l_fmt=VIDEO_PALETTE_YUV422;
 			vid_mmap.format=viddev->v4l_fmt;
 			/* Try again... */
 			if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-				syslog(LOG_DEBUG, "Failed with YUV422, trying RGB24 palette\n");
+				const char msg[]="Failed with YUV422, trying RGB24 palette";
+				printf("[%d] %s\n", cnt->threadnr, msg);
+				syslog(LOG_DEBUG, "[%d] %s", cnt->threadnr, msg);
 				viddev->v4l_fmt=VIDEO_PALETTE_RGB24;
 				vid_mmap.format=viddev->v4l_fmt;
 				/* Try again... */
 				if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-					syslog(LOG_ERR, "ioctl VIDIOCMCAPTURE: %m");
+					printf("[%d] ioctl VIDIOCMCAPTURE: %m\n", cnt->threadnr);
+					syslog(LOG_ERR, "[%d] ioctl VIDIOCMCAPTURE: %m", cnt->threadnr);
 					return (NULL);
 				}
 			}
@@ -320,7 +368,7 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	return map;
 }
 
-static char *v4l_next (struct video_dev *viddev, unsigned char *map, int width, int height)
+static char *v4l_next (struct video_dev *viddev, unsigned char *map, int width, int height, struct context *cnt)
 {
 	int dev=viddev->fd;
 	int frame=viddev->v4l_curbuffer;
@@ -352,14 +400,18 @@ static char *v4l_next (struct video_dev *viddev, unsigned char *map, int width,
 		vid_mmap.frame=viddev->v4l_curbuffer;
 
 		if (ioctl(dev, VIDIOCMCAPTURE, &vid_mmap) == -1) {
-			syslog(LOG_ERR, "mcapture error in proc %d: %m", getpid());
+			const char msg[]="mcapture error in proc ";
+			printf("[%d] %s%d: %m\n", cnt->threadnr, msg, getpid());
+			syslog(LOG_ERR, "[%d] %s%d: %m", cnt->threadnr, msg, getpid());
 			sigprocmask (SIG_UNBLOCK, &old, NULL);
 			return (NULL);
 		}
 
 		vid_mmap.frame=frame;
 		if (ioctl(dev, VIDIOCSYNC, &vid_mmap.frame) == -1) {
-			syslog(LOG_ERR, "sync error in proc %d: %m", getpid());
+			const char msg[]="sync error in proc ";
+			printf("[%d] %s%d: %m\n", cnt->threadnr, msg, getpid());
+			syslog(LOG_ERR, "[%d] %s%d: %m", cnt->threadnr, msg, getpid());
 			sigprocmask (SIG_UNBLOCK, &old, NULL);
 		}
 		
@@ -381,7 +433,8 @@ static char *v4l_next (struct video_dev *viddev, unsigned char *map, int width,
 	return map;
 }
 
-void v4l_set_input (struct video_dev *viddev, unsigned char *map, int width, int height, int input, int norm, int skip, unsigned long freq, struct context *cnt)
+void v4l_set_input (struct video_dev *viddev, unsigned char *map, int width, int height, int input,
+                    int norm, int skip, unsigned long freq, int tuner_number, struct context *cnt)
 {
 	int dev=viddev->fd;
 	int i;
@@ -389,10 +442,13 @@ void v4l_set_input (struct video_dev *viddev, unsigned char *map, int width, int
 	struct video_tuner vid_tuner;
 	unsigned long frequnits = freq;
 	
-	if (input != viddev->input || width != viddev->width || height!=viddev->height || freq!=viddev->freq) {
+	if (input != viddev->input || width != viddev->width || height!=viddev->height ||
+	    freq!=viddev->freq || tuner_number!=viddev->tuner_number) {
 		if (freq) {
+			vid_tuner.tuner = tuner_number;
 			if (ioctl (dev, VIDIOCGTUNER, &vid_tuner)==-1) {
-				syslog(LOG_ERR, "ioctl (VIDIOCGTUNER): %m");
+				printf("[%d] ioctl (VIDIOCGTUNER): %m\n", cnt->threadnr);
+				syslog(LOG_ERR, "[%d] ioctl (VIDIOCGTUNER): %m", cnt->threadnr);
 			} else {
 				if (vid_tuner.flags & VIDEO_TUNER_LOW) {
 					frequnits=freq*16; /* steps of 1/16 KHz */
@@ -400,21 +456,25 @@ void v4l_set_input (struct video_dev *viddev, unsigned char *map, int width, int
 					frequnits=(freq*10)/625;
 				}
 				if (ioctl(dev, VIDIOCSFREQ, &frequnits)==-1) {
-					syslog(LOG_ERR, "ioctl (VIDIOCSFREQ): %m");
+					printf("[%d] ioctl (VIDIOCSFREQ): %m\n", cnt->threadnr);
+					syslog(LOG_ERR, "[%d] ioctl (VIDIOCSFREQ): %m", cnt->threadnr);
 					return;
 				}
 			}
 		}
 
 		vid_chnl.channel = input;
+		
 		if (ioctl (dev, VIDIOCGCHAN, &vid_chnl) == -1) {
-			syslog(LOG_ERR, "ioctl (VIDIOCGCHAN): %m");
+			printf("[%d] ioctl (VIDIOCGCHAN): %m\n", cnt->threadnr);
+			syslog(LOG_ERR, "[%d] ioctl (VIDIOCGCHAN): %m", cnt->threadnr);
 		} else {
 			//DEBUG CODE syslog(LOG_DEBUG, "input: %d", input);
 			vid_chnl.channel = input;
 			vid_chnl.norm = norm;
 			if (ioctl (dev, VIDIOCSCHAN, &vid_chnl) == -1) {
-				syslog(LOG_ERR, "ioctl (VIDIOCSCHAN): %m");
+				printf("[%d] ioctl (VIDIOCSCHAN): %m\n", cnt->threadnr);
+				syslog(LOG_ERR, "[%d] ioctl (VIDIOCSCHAN): %m", cnt->threadnr);
 				return;
 			}
 		}
@@ -423,16 +483,17 @@ void v4l_set_input (struct video_dev *viddev, unsigned char *map, int width, int
 		viddev->width=width;
 		viddev->height=height;
 		viddev->freq=freq;
+		viddev->tuner_number=tuner_number;
 		/* skip a few frames if needed */
 		for (i=0; i<skip; i++)
-			v4l_next (viddev, map, width, height);
+			v4l_next (viddev, map, width, height, cnt);
 	} else {
 		/* No round robin - we only adjust picture controls */
 		v4l_picture_controls(viddev, cnt);
 	}
 }
 
-static int v4l_open_vidpipe(void)
+static int v4l_open_vidpipe(struct context *cnt)
 {
 	int pipe = -1;
 	char pipepath[255];
@@ -442,13 +503,17 @@ static int v4l_open_vidpipe(void)
 	struct utsname uts;
 
 	if (uname(&uts) < 0) {
-		syslog(LOG_ERR, "Unable to execute uname: %m");
+		const char msg[]="Unable to execute uname: ";
+		printf("[%d] %s%m\n", cnt->threadnr, msg);
+		syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 		return -1;
 	}
 	major = strtok(uts.release, ".");
 	minor = strtok(NULL, ".");
 	if ((major == NULL) || (minor == NULL) || (strcmp(major, "2"))) {
-		syslog(LOG_ERR, "Unable to decipher OS version\n");
+		const char msg[]="Unable to decipher OS version";
+		printf("[%d] %s\n", cnt->threadnr, msg);
+		syslog(LOG_ERR, "[%d] %s", cnt->threadnr, msg);
 		return -1;
 	}
 	if (strcmp(minor, "5") < 0) {
@@ -461,7 +526,9 @@ static int v4l_open_vidpipe(void)
 
 		vloopbacks=fopen("/proc/video/vloopback/vloopbacks", "r");
 		if (!vloopbacks) {
-			syslog(LOG_ERR, "Failed to open '/proc/video/vloopback/vloopbacks': %m");
+			const char msg[]="Failed to open '/proc/video/vloopback/vloopbacks': ";
+			printf("[%d] %s%m\n", cnt->threadnr, msg);
+			syslog(LOG_ERR, "[%d] %s%m", cnt->threadnr, msg);
 			return -1;
 		}
 		/* Read vloopback version*/
@@ -500,7 +567,9 @@ static int v4l_open_vidpipe(void)
 		int tnum;
 
 		if ((dir=opendir(prefix))== NULL) {
-			syslog(LOG_ERR, "Failed to open '%s': %m", prefix);
+			const char msg[]="Failed to open ";
+			printf("[%d] %s'%s': %m\n", cnt->threadnr, msg, prefix);
+			syslog(LOG_ERR, "[%d] %s'%s': %m", cnt->threadnr, msg, prefix);
 			return -1;
 		}
 		while ((dirp=readdir(dir)) != NULL) {
@@ -548,20 +617,20 @@ static int v4l_open_vidpipe(void)
 		}
 		closedir(dir);
 		if (pipe >= 0) {
-			fprintf(stderr,"Opened input of %s\n", pipepath);
+			printf("[%d] Opened input of %s\n", cnt->threadnr, pipepath);
 		}
 	}
 	return pipe;
 }
 
-static int v4l_startpipe (char *devname, int width, int height, int type)
+static int v4l_startpipe (char *devname, int width, int height, int type, struct context *cnt)
 {
 	int dev;
 	struct video_picture vid_pic;
 	struct video_window vid_win;
 
 	if (!strcmp(devname, "-")) {
-		dev=v4l_open_vidpipe();
+		dev=v4l_open_vidpipe(cnt);
 	} else {
 		dev=open(devname, O_RDWR);
 	}
@@ -569,22 +638,26 @@ static int v4l_startpipe (char *devname, int width, int height, int type)
 		return(-1);
 
 	if (ioctl(dev, VIDIOCGPICT, &vid_pic)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCGPICT: %m");
+		printf("[%d] ioctl VIDIOCGPICT: %m\n", cnt->threadnr);
+		syslog(LOG_ERR, "[%d] ioctl VIDIOCGPICT: %m", cnt->threadnr);
 		return(-1);
 	}
 	vid_pic.palette=type;
 	if (ioctl(dev, VIDIOCSPICT, &vid_pic)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCSPICT: %m");
+		printf("[%d] ioctl VIDIOCSPICT: %m\n", cnt->threadnr);
+		syslog(LOG_ERR, "[%d] ioctl VIDIOCSPICT: %m", cnt->threadnr);
 		return(-1);
 	}
 	if (ioctl(dev, VIDIOCGWIN, &vid_win)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCGWIN: %m");
+		printf("[%d] ioctl VIDIOCGWIN: %m\n", cnt->threadnr);
+		syslog(LOG_ERR, "[%d] ioctl VIDIOCGWIN: %m", cnt->threadnr);
 		return(-1);
 	}
 	vid_win.height=height;
 	vid_win.width=width;
 	if (ioctl(dev, VIDIOCSWIN, &vid_win)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCSWIN: %m");
+		printf("[%d] ioctl VIDIOCSWIN: %m\n", cnt->threadnr);
+		syslog(LOG_ERR, "[%d] ioctl VIDIOCSWIN: %m", cnt->threadnr);
 		return(-1);
 	}
 	return dev;
@@ -599,8 +672,11 @@ static int v4l_putpipe (int dev, char *image, int size)
 	Wrappers calling the actual capture routines
  *****************************************************************************/
 
+#endif /*WITHOUT_V4L*/
 /* big lock for vid_start */
 pthread_mutex_t vid_mutex;
+#ifndef WITHOUT_V4L
+
 /* structure used for per device locking */
 struct video_dev **viddevs=NULL;
 
@@ -623,7 +699,7 @@ void vid_close(void)
 		while(viddevs[++i]) {
 			close(viddevs[i]->fd);
 		}
-		fprintf(stderr,"Closing devices ...\n");
+		//fprintf(stderr,"Closing devices ...\n");
 	}
 }
 
@@ -631,11 +707,13 @@ void vid_cleanup(void)
 {
 	int i=-1;
 	if (viddevs) {
-		while(viddevs[++i])
+		while(viddevs[++i]){
+			munmap(viddevs[i]->v4l_buffers[0],viddevs[i]->size_map);
 			free(viddevs[i]);
+		}
 		free(viddevs);
 		viddevs=NULL;
-		fprintf(stderr,"Cleaning viddevs ...\n");
+		//fprintf(stderr,"Cleaning viddevs ...\n");
 	}
 }
 #endif /*WITHOUT_V4L*/
@@ -645,13 +723,14 @@ int vid_start (struct context *cnt)
 	struct config *conf=&cnt->conf;
 	int dev=-1;
 
-	if (conf->netcam_url)
+	if (conf->netcam_url) {
 		return netcam_start(cnt);
+	}
 
 #ifndef WITHOUT_V4L
 	{
 		int i=-1;
-		int width, height, input, norm;
+		int width, height, input, norm, tuner_number;
 		unsigned long frequency;
 
 		/* We use width and height from conf in this function. They will be assigned
@@ -663,6 +742,7 @@ int vid_start (struct context *cnt)
 		input = conf->input;
 		norm = conf->norm;
 		frequency = conf->frequency;
+		tuner_number = conf->tuner_number;
 		
 		pthread_mutex_lock(&vid_mutex);
 
@@ -675,7 +755,7 @@ int vid_start (struct context *cnt)
 
 		while (viddevs[++i]) {
 			if (!strcmp(conf->video_device, viddevs[i]->video_device)) {
-				pthread_mutex_unlock(&vid_mutex);
+			  	int fd;
 				cnt->imgs.type=viddevs[i]->v4l_fmt;
 				switch (cnt->imgs.type) {
 					case VIDEO_PALETTE_GREY:
@@ -690,7 +770,9 @@ int vid_start (struct context *cnt)
 						cnt->imgs.size=(width*height*3)/2;
 						break;
 				}
-				return viddevs[i]->fd;
+				fd=viddevs[i]->fd;
+				pthread_mutex_unlock(&vid_mutex);
+				return fd;
 			}
 		}
 
@@ -704,8 +786,8 @@ int vid_start (struct context *cnt)
 
 		dev=open(conf->video_device, O_RDWR);
 		if (dev <0) {
-			printf("Failed to open video device %s\n", conf->video_device);
-			syslog(LOG_ERR, "failed to open video device %s: %m", conf->video_device);
+			printf("[%d] Failed to open video device %s\n", cnt->threadnr, conf->video_device);
+			syslog(LOG_ERR, "[%d] Failed to open video device %s: %m", cnt->threadnr, conf->video_device);
 			exit(1);
 		}
 
@@ -715,6 +797,7 @@ int vid_start (struct context *cnt)
 		viddevs[i]->height=height;
 		viddevs[i]->width=width;
 		viddevs[i]->freq=frequency;
+		viddevs[i]->tuner_number=tuner_number;
 		
 		/* We set brightness, contrast, saturation and hue = 0 so that they only get
 		 * set if the config is not zero.
@@ -727,7 +810,7 @@ int vid_start (struct context *cnt)
 
 		viddevs[i]->v4l_fmt=VIDEO_PALETTE_YUV420P;
 
-		if (!v4l_start (viddevs[i], width, height, input, norm, frequency)) {
+		if (!v4l_start (viddevs[i], width, height, input, norm, frequency, tuner_number, cnt)) {
 			pthread_mutex_unlock(&vid_mutex);
 			return -1;
 		}
@@ -759,7 +842,8 @@ char *vid_next (struct context *cnt, unsigned char *map)
 
 	if (conf->netcam_url) {
 		int retries=0;
-		while ((!cnt->finish) && (!ret) && (retries++<100)) ret = netcam_next(cnt, map);
+		while ((!cnt->finish) && (!ret) && (retries++<100))
+			ret = netcam_next(cnt, map);
 		return ret;
 	}
 #ifndef WITHOUT_V4L
@@ -786,8 +870,9 @@ char *vid_next (struct context *cnt, unsigned char *map)
 			cnt->switched=1;
 		}
 
-		v4l_set_input (viddevs[i], map, width, height, conf->input, conf->norm, conf->roundrobin_skip, conf->frequency, cnt);
-		ret=v4l_next (viddevs[i], map, width, height);
+		v4l_set_input (viddevs[i], map, width, height, conf->input, conf->norm,
+		               conf->roundrobin_skip, conf->frequency, conf->tuner_number, cnt);
+		ret=v4l_next (viddevs[i], map, width, height, cnt);
 
 		if (--viddevs[i]->frames <= 0) {
 			viddevs[i]->owner=-1;
@@ -804,9 +889,9 @@ char *vid_next (struct context *cnt, unsigned char *map)
 }
 
 #ifndef WITHOUT_V4L
-int vid_startpipe (char *devname, int width, int height, int type)
+int vid_startpipe (char *devname, int width, int height, int type, struct context *cnt)
 {
-	return v4l_startpipe (devname, width, height, type);
+	return v4l_startpipe (devname, width, height, type, cnt);
 }
 
 int vid_putpipe (int dev, char *image, int size)
