@@ -194,6 +194,7 @@ static int set_chroma( struct video_dev *viddev, int new_chroma )
 	new_chroma = camparam_normalize( CHROMA, new_chroma, &ioctlval );
                                                                                                                              
 	if( ioctl( viddev->fd_tuner, BT848_SCSAT, &ioctlval ) < 0 ) {
+		printf( "Error setting CHROMA: %s\n", strerror( errno ));
 		syslog( LOG_ERR, "Error setting CHROMA: %s", strerror( errno ));
 		return -1;
 	}
@@ -210,6 +211,7 @@ static int set_contrast( struct video_dev *viddev, int new_contrast )
 	new_contrast = camparam_normalize( CONTR, new_contrast, &ioctlval );
   
 	if( ioctl( viddev->fd_tuner, BT848_SCONT, &ioctlval ) < 0 ) {
+		printf( "Error setting contrast: %s\n",strerror( errno ));
 		syslog( LOG_ERR, "Error setting contrast: %s",strerror( errno ));
 		return -1;
 	}
@@ -228,9 +230,11 @@ static int set_channel( struct video_dev *viddev, int new_channel )
                                                                                                                              
 	ioctlval = new_channel;
 	if( ioctl( viddev->fd_tuner, TVTUNER_SETCHNL, &ioctlval ) < 0 ) {
+		printf( "Error channel->%d : %s\n", ioctlval, strerror( errno ) );
 		syslog( LOG_ERR, "Error channel->%d : %s", ioctlval, strerror( errno ) );
 		return -1;
 	} else {
+		printf("channel set to %d\n", ioctlval);
 		syslog(LOG_ERR, "channel set to %d", ioctlval);
 	}
                                                                                                                              
@@ -242,7 +246,7 @@ static int set_channel( struct video_dev *viddev, int new_channel )
 
 /* set frequency to tuner */
 
-static int set_freq (struct video_dev *viddev,unsigned long freq)
+static int set_freq (struct video_dev *viddev,unsigned long freq, struct context *cnt)
 {
 	int tuner_fd = viddev->fd_tuner;
 	int old_audio;
@@ -250,12 +254,14 @@ static int set_freq (struct video_dev *viddev,unsigned long freq)
 	/* HACK maybe not need it , but seems that is needed to mute before changing frequency */
 
 	if ( ioctl( tuner_fd, BT848_GAUDIO, &old_audio ) < 0 ) {
-		syslog( LOG_ERR , "BT848_GAUDIO" );
+		printf("[%d] BT848_GAUDIO\n", cnt->threadnr);
+		syslog( LOG_ERR , "[%d] BT848_GAUDIO", cnt->threadnr);
 		return -1;
 	}
 	
 	if (ioctl(tuner_fd, TVTUNER_SETFREQ, &freq) < 0){
-		syslog(LOG_ERR,"Tuning (TVTUNER_SETFREQ) failed, reason [%s] freq [%lu]",strerror(errno),freq);
+		printf("[%d] Tuning (TVTUNER_SETFREQ) failed, reason [%s] freq [%lu]\n", cnt->threadnr, strerror(errno),freq);
+		syslog(LOG_ERR,"[%d] Tuning (TVTUNER_SETFREQ) failed, reason [%s] freq [%lu]", cnt->threadnr, strerror(errno),freq);
 		return -1;	
 	}
 
@@ -263,7 +269,8 @@ static int set_freq (struct video_dev *viddev,unsigned long freq)
 	if ( old_audio ){
 		old_audio = AUDIO_MUTE;
 		if ( ioctl(tuner_fd , BT848_SAUDIO, &old_audio ) < 0 ) {
-			syslog ( LOG_ERR,"BT848_SAUDIO [%i]", old_audio );
+			printf ( "[%d] BT848_SAUDIO [%i]\n", cnt->threadnr, old_audio );
+			syslog ( LOG_ERR,"[%d] BT848_SAUDIO [%i]", cnt->threadnr, old_audio );
 			return -1;
 		}
 	}
@@ -280,7 +287,7 @@ static int set_freq (struct video_dev *viddev,unsigned long freq)
 	VBI ?! (METEOR_INPUT_DEV_SVIDEO)
 */
 
-static int set_input( struct video_dev *viddev , int input )
+static int set_input( struct video_dev *viddev , int input , struct context *cnt)
 {
 	int actport;
 	int portdata[] = { METEOR_INPUT_DEV0, METEOR_INPUT_DEV1,
@@ -288,22 +295,26 @@ static int set_input( struct video_dev *viddev , int input )
 			METEOR_INPUT_DEV_SVIDEO  };
                                                                                                                              
 	if( input >= array_elem( portdata ) ){
-		syslog( LOG_ERR, "Port %d out of range (0-4)", input );
+		printf( "[%d] Port %d out of range (0-4)\n", cnt->threadnr, input );
+		syslog( LOG_ERR, "[%d] Port %d out of range (0-4)", cnt->threadnr, input );
 		input = IN_DEFAULT;
 	}
 														     
 	actport = portdata[ input ];
 	if( ioctl( viddev->fd_bktr, METEORSINPUT, &actport ) < 0 ){
 		if( input != IN_DEFAULT ) {
-			syslog( LOG_ERR, "Port %d invalid -- Trying default of %d",input, IN_DEFAULT );
+			printf("[%d] Port %d invalid -- Trying default of %d\n",cnt->threadnr, input, IN_DEFAULT );
+			syslog( LOG_ERR, "[%d] Port %d invalid -- Trying default of %d",cnt->threadnr, input, IN_DEFAULT );
 			input = IN_DEFAULT;
 			actport = portdata[ input ];
 			if( ioctl( viddev->fd_bktr, METEORSINPUT, &actport ) < 0 ) {
-				syslog( LOG_ERR, "Port %d init: %s", input, strerror( errno ));
+				printf( "[%d] Port %d init: %s\n", cnt->threadnr, input, strerror( errno ));
+				syslog( LOG_ERR, "[%d] Port %d init: %s", cnt->threadnr, input, strerror( errno ));
 				return -1;
 			}
 		} else {
-			syslog( LOG_ERR, "Port %d init: %s", input , strerror( errno ));
+			printf( "[%d] Port %d init: %s\n", cnt->threadnr, input , strerror( errno ));
+			syslog( LOG_ERR, "[%d] Port %d init: %s", cnt->threadnr, input , strerror( errno ));
 			return -1;
 		}
 	}
@@ -311,7 +322,7 @@ static int set_input( struct video_dev *viddev , int input )
 	return 0;
 }
 
-static int set_geometry( struct video_dev *viddev, int width , int height , int format)
+static int set_geometry( struct video_dev *viddev, int width , int height , int format, struct context *cnt)
 {
 	struct meteor_geomet geom;
 														     
@@ -322,7 +333,8 @@ static int set_geometry( struct video_dev *viddev, int width , int height , int
 	geom.frames  = 1;
                                                                                                                           
   	if( ioctl( viddev->fd_bktr, METEORSETGEO, &geom ) < 0 ) {
-		syslog( LOG_ERR, "Couldn't set the geometry: %s",strerror( errno ) );
+		printf( "[%d] Couldn't set the geometry: %s\n", cnt->threadnr, strerror( errno ) );
+		syslog( LOG_ERR, "[%d] Couldn't set the geometry: %s", cnt->threadnr, strerror( errno ) );
 		return -1;
   	}
                                                                                                                              
@@ -333,20 +345,22 @@ static int set_geometry( struct video_dev *viddev, int width , int height , int
    set input format ( NTSC , PAL , SECAM , etc ... ) not all supported yet FIXME
 */
 
-static int set_input_format(struct video_dev *viddev, int newformat) 
+static int set_input_format(struct video_dev *viddev, int newformat, struct context *cnt) 
 {
 	int format;
 	int input_format[] = { NORM_PAL, NORM_NTSC, NORM_SECAM };
  
 	if( newformat >= array_elem( input_format ) ){
-		syslog( LOG_ERR, "Input format %d out of range (0-2)", newformat );
+		printf( "[%d] Input format %d out of range (0-2)\n", cnt->threadnr, newformat );
+		syslog( LOG_ERR, "[%d] Input format %d out of range (0-2)", cnt->threadnr, newformat );
 		format = NORM_DEFAULT;newformat = 0;
 	}else format = input_format[newformat]; 
 
 	/* FIXME should be BT848SFMT for new BKTR INTERFACES */
 
 	if( ioctl( viddev->fd_bktr, METEORSFMT, &format ) < 0 ) {
-		syslog( LOG_ERR, "Couldn't set the input format : %s",strerror( errno ) );
+		printf( "[%d] Couldn't set the input format : %s\n", cnt->threadnr, strerror( errno ) );
+		syslog( LOG_ERR, "[%d] Couldn't set the input format : %s", cnt->threadnr, strerror( errno ) );
 		return -1;
 	}
 	return 0;
@@ -359,6 +373,7 @@ static int set_brightness( struct video_dev *viddev, int new_bright )
                                                                                                                              
 	new_bright = camparam_normalize( BRIGHT, new_bright, &ioctlval );
 	if( ioctl( viddev->fd_tuner, BT848_SBRIG, &ioctlval ) < 0 ) {
+		printf( "Error brightness->%d : %s\n", ioctlval,strerror( errno ) );
 		syslog( LOG_ERR, "Error brightness->%d : %s", ioctlval,strerror( errno ) );
 		return -1;
 	}
@@ -380,6 +395,7 @@ statict int setup_pixelformat( int bktr )
 		if( ioctl( bktr, METEORGSUPPIXFMT, &p ) < 0 ){
 			if( errno == EINVAL )
 				break;
+			printf("Error getting pixformat %d: %s\n", i,strerror( errno ) );
 			syslog(LOG_ERR, "Error getting pixformat %d: %s", i,strerror( errno ) );
 			return -1;
 		}
@@ -430,6 +446,7 @@ statict int setup_pixelformat( int bktr )
 			if( p.type == METEOR_PIXTYPE_RGB && p.Bpp == 3 ){
 				// Found a good pixeltype -- set it up 
 				if( ioctl( bktr, METEORSACTPIXFMT, &i ) < 0 ){
+					printf( "Error setting pixformat METEOR_PIXTYPE_RGB Bpp == 3 : %s\n",strerror( errno ));
 					syslog( LOG_ERR, "Error setting pixformat METEOR_PIXTYPE_RGB Bpp == 3 : %s",strerror( errno ));
 				// Not immediately fatal 
 				} //else return 0;
@@ -440,6 +457,7 @@ statict int setup_pixelformat( int bktr )
 			if( p.type == METEOR_PIXTYPE_YUV_PACKED ){
 				// Found a good pixeltype -- set it up
 				if( ioctl( bktr, METEORSACTPIXFMT, &i ) < 0 ){
+					printf( "Error setting pixformat METEOR_PIXTYPE_YUV_PACKED: %s\n", strerror( errno ));
 					syslog( LOG_ERR, "Error setting pixformat METEOR_PIXTYPE_YUV_PACKED: %s", strerror( errno ));
 				// Not immediately fatal
 				} //else return 0;
@@ -469,7 +487,7 @@ statict int setup_pixelformat( int bktr )
  - set_capture_mode
 
 */
-static char *v4l_start (struct video_dev *viddev, int width, int height, int input, int norm, unsigned long freq)
+static char *v4l_start (struct video_dev *viddev, int width, int height, int input, int norm, unsigned long freq, struct context *cnt)
 {
 	int dev_bktr=viddev->fd_bktr;
 	//int dev_tunner=viddev->fd_tuner;
@@ -479,25 +497,26 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	int single  = METEOR_CAP_SINGLE;
 	char *map;
 
-	if ( ((freq <= 0) && (input == IN_TV)) || ( set_input(viddev,input) == -1) ){
-		 syslog(LOG_ERR,"Error!  Frequency [%lu] Source input [%i]",freq,input);
+	if ( ((freq <= 0) && (input == IN_TV)) || ( set_input(viddev,input,cnt) == -1) ){
+		 printf("[%d] Error!  Frequency [%lu] Source input [%i]\n",cnt->threadnr,freq,input);
+		 syslog(LOG_ERR,"[%d] Error!  Frequency [%lu] Source input [%i]",cnt->threadnr,freq,input);
 		 return (NULL);
 	}
  
 	/* if we have choose the tuner is needed to setup the frequency */
 	if ( input == IN_TV ){
-		if (set_freq (viddev,freq) == -1){
+		if (set_freq (viddev,freq, cnt) == -1){
 			return (NULL);
 		}
 	}
 	
 	/* FIXME if we set as input tuner , we need to set option for tuner not for bktr */
 
-	if ( set_input_format(viddev,norm) == -1 ){
+	if ( set_input_format(viddev,norm,cnt) == -1 ){
 		return (NULL);
 	}
 
-	if ( set_geometry(viddev,width,height,norm) == -1){
+	if ( set_geometry(viddev,width,height,norm, cnt) == -1){
 		return (NULL);
         }
 
@@ -516,7 +535,7 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	 TODO missing implementation
 		set_channelset(viddev);
 		set_channel(viddev);
-		if (set_freq (viddev,freq) == -1){
+		if (set_freq (viddev,freq, cnt) == -1){
                         return (NULL);
                 }
 	*/
@@ -534,7 +553,7 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
         map = mmap((caddr_t)0,viddev->v4l_bufsize,PROT_READ|PROT_WRITE,MAP_SHARED, dev_bktr, (off_t)0);
 
 	if ((unsigned char *)-1 == (unsigned char *)map) {
-		printf("mmap failed [%s]\n",strerror(errno));
+		printf("[%d] mmap failed [%s]\n",cnt->threadnr,strerror(errno));
                 return (NULL);
         }
 
@@ -556,8 +575,10 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	
 	/* capture */	
 	if (ioctl(dev_bktr, METEORCAPTUR, &single) < 0){
-		syslog( LOG_ERR, "Error capturing: %s",
-                       strerror( errno ));
+		printf( "[%d] Error capturing: %s\n",
+                       cnt->threadnr, strerror( errno ));
+		syslog( LOG_ERR, "[%d] Error capturing: %s",
+                       cnt->threadnr, strerror( errno ));
 	}
  	
 	
@@ -565,7 +586,7 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
 	/* FIXME*/
 	switch (viddev->v4l_fmt) {
                 case VIDEO_PALETTE_YUV420P:
-			printf("VIDEO_PALETTE_YUV420P palette setting bufsize \n");
+			printf("[%d] VIDEO_PALETTE_YUV420P palette setting bufsize \n",cnt->threadnr);
                         viddev->v4l_bufsize=(width*height*3)/2;
                         break;
                 case VIDEO_PALETTE_YUV422:
@@ -587,7 +608,7 @@ static char *v4l_start (struct video_dev *viddev, int width, int height, int inp
   Read the next image from device 
 */
 
-static char *v4l_next (struct video_dev *viddev, char *map, int width, int height)
+static char *v4l_next (struct video_dev *viddev, char *map, int width, int height, struct context *cnt)
 {
 	int dev_bktr=viddev->fd_bktr;
 	char *cap_map=NULL;
@@ -617,8 +638,8 @@ static char *v4l_next (struct video_dev *viddev, char *map, int width, int heigh
 
 		 /* capture */
         	if (ioctl(dev_bktr, METEORCAPTUR, &single) < 0){
-                	syslog( LOG_ERR, "Error capturing in proc %d: %s",getpid(),
-                       	strerror( errno ));
+                	printf( "[%d] Error capturing in proc %d: %s\n",cnt->threadnr,getpid(),strerror( errno ));
+                	syslog( LOG_ERR, "[%d] Error capturing in proc %d: %s",cnt->threadnr,getpid(),strerror( errno ));
 			sigprocmask (SIG_UNBLOCK, &old, NULL);
 			return (NULL);
         	}
@@ -645,37 +666,39 @@ static char *v4l_next (struct video_dev *viddev, char *map, int width, int heigh
 
 /* set input & freq if needed FIXME not allowed use Tuner yet */
 
-void v4l_set_input (struct video_dev *viddev, char *map, int width, int height, int input, int norm, int skip, unsigned long freq)
+void v4l_set_input (struct video_dev *viddev, char *map, int width, int height, int input, int norm, int skip, unsigned long freq, struct context *cnt)
 {
 	int i;
 	unsigned long frequnits = freq;
 
 
 	if (input != viddev->input || width != viddev->width || height!=viddev->height || freq!=viddev->freq) {
-		syslog(LOG_ERR, " v4l_set_input really needed ?");
+		printf("[%d] v4l_set_input really needed ?\n",cnt->threadnr);
+		syslog(LOG_ERR, "[%d] v4l_set_input really needed ?",cnt->threadnr);
 
-		if ( set_input(viddev,input) == -1){
+		if ( set_input(viddev,input,cnt) == -1){
 			return;
 		}
 
-        	if ( set_input_format(viddev,norm) == -1 ){
+        	if ( set_input_format(viddev,norm,cnt) == -1 ){
 			return;
 		}
 		
 		if (( input == IN_TV ) && (frequnits > 0)){
-			if (set_freq (viddev,freq) == -1){
+			if (set_freq (viddev,freq, cnt) == -1){
 				return ;
 			}
 		}
  
 		/* FIXME
         	if ( setup_pixelformat(viddev) == -1){
+			printf("ioctl (VIDIOCSFREQ): %m\n");
 			syslog(LOG_ERR, "ioctl (VIDIOCSFREQ): %m");
 			return;
        		}
 		*/
 
-        	if ( set_geometry(viddev,width,height,norm) == -1){
+        	if ( set_geometry(viddev,width,height,norm, cnt) == -1){
 			return;
 		}
 		
@@ -686,102 +709,11 @@ void v4l_set_input (struct video_dev *viddev, char *map, int width, int height,
 
 		/* skip a few frames if needed */
 		for (i=0; i<skip; i++){
-			v4l_next (viddev, map, width, height);
+			v4l_next (viddev, map, width, height, cnt);
 		}
 	}
 }
 
-/* Not implemented yet
-static int v4l_open_vidpipe(void)
-{
-	int pipe;
-	FILE *vloopbacks;
-	char pipepath[255];
-	char buffer[255];
-	char *loop;
-	char *input;
-	char *istatus;
-	char *output;
-	char *ostatus;
-	
-	vloopbacks=fopen("/proc/video/vloopback/vloopbacks", "r");
-	if (!vloopbacks) {
-		syslog(LOG_ERR, "Failed to open '/proc/video/vloopback/vloopbacks': %m");
-		return -1;
-	}
-	// Read vloopback version
-	fgets(buffer, 255, vloopbacks);
-	printf("\t%s", buffer);
-	// Read explaination line 
-	fgets(buffer, 255, vloopbacks);
-	while (fgets(buffer, 255, vloopbacks)) {
-		if (strlen(buffer)>1) {
-			buffer[strlen(buffer)-1]=0;
-			loop=strtok(buffer, "\t");
-			input=strtok(NULL, "\t");
-			istatus=strtok(NULL, "\t");
-			output=strtok(NULL, "\t");
-			ostatus=strtok(NULL, "\t");
-			if (istatus[0]=='-') {
-				sprintf(pipepath, "/dev/%s", input);
-				pipe=open(pipepath, O_RDWR);
-				if (pipe>=0) {
-					printf("\tInput: /dev/%s\n", input);
-					printf("\tOutput: /dev/%s\n", output);
-					return pipe;
-				}
-			}
-		}
-	}
- 
-	return -1;
-}
-*/
-
-static int v4l_startpipe (char *devname, int width, int height, int type)
-{
-/* Not implemented
-	int dev;
-	struct video_picture vid_pic;
-	struct video_window vid_win;
-
-	if (!strcmp(devname, "-")) {
-		dev=v4l_open_vidpipe();
-	} else {
-		dev=open(devname, O_RDWR);
-	}
-	if (dev < 0)
-		return(-1);
-
-	if (ioctl(dev, VIDIOCGPICT, &vid_pic)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCGPICT: %m");
-		return(-1);
-	}
-	vid_pic.palette=type;
-	if (ioctl(dev, VIDIOCSPICT, &vid_pic)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCSPICT: %m");
-		return(-1);
-	}
-	if (ioctl(dev, VIDIOCGWIN, &vid_win)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCGWIN: %m");
-		return(-1);
-	}
-	vid_win.height=height;
-	vid_win.width=width;
-	if (ioctl(dev, VIDIOCSWIN, &vid_win)== -1) {
-		syslog(LOG_ERR, "ioctl VIDIOCSWIN: %m");
-		return(-1);
-	}
-	return dev;
-*/
- return -1;
-}
-
-static int v4l_putpipe (int dev, char *image, int size)
-{
-	return write(dev, image, size);
-}
-
 /*****************************************************************************
 	Wrappers calling the actual capture routines
  *****************************************************************************/
@@ -821,8 +753,9 @@ void vid_close(void)
 		while(viddevs[++i]) {
 		/*	stopcap = METEOR_CAP_STOP_CONT;
   			if( ioctl( viddevs[++i]->fd_bktr, METEORCAPTUR, &stopcap ) < 0 )
+    			printf("vid_close: %s\n", strerror( errno ));
     			syslog(LOG_ERR, "vid_close: %s", strerror( errno ));
-                */                                                                                                     
+                */
 			close(viddevs[i]->fd_bktr);
 			close(viddevs[i]->fd_tuner);
 		}
@@ -833,8 +766,10 @@ void vid_cleanup(void)
 {
 	int i=-1;
 	if (viddevs) {
-		while(viddevs[++i])
+		while(viddevs[++i]){
+			munmap(viddevs[i]->v4l_buffers[0],viddevs[i]->v4l_bufsize);
 			free(viddevs[i]);
+		}
 		free(viddevs);
 		viddevs=NULL;
 	}
@@ -878,9 +813,9 @@ int vid_start (struct context *cnt)
 
 		while (viddevs[++i]) {
 			if (!strcmp(conf->video_device, viddevs[i]->video_device)) {
-				pthread_mutex_unlock(&vid_mutex);
+			  	int fd;
 				cnt->imgs.type=viddevs[i]->v4l_fmt;
-				printf("vid_start cnt->imgs.type [%i]\n",cnt->imgs.type);
+				printf("[%d] vid_start cnt->imgs.type [%i]\n",cnt->threadnr,cnt->imgs.type);
 				switch (cnt->imgs.type) {
 					case VIDEO_PALETTE_GREY:
 						cnt->imgs.motionsize=width*height;
@@ -890,12 +825,14 @@ int vid_start (struct context *cnt)
 					case VIDEO_PALETTE_YUV422:
 						cnt->imgs.type=VIDEO_PALETTE_YUV420P;
 					case VIDEO_PALETTE_YUV420P:
-						printf("VIDEO_PALETTE_YUV420P setting imgs.size and imgs.motionsize\n");
+						printf("[%d] VIDEO_PALETTE_YUV420P setting imgs.size and imgs.motionsize\n",cnt->threadnr);
 						cnt->imgs.motionsize=width*height;
 						cnt->imgs.size=(width*height*3)/2;
 					break;
 				}
-				return viddevs[i]->fd_bktr; // FIXME return fd_tuner ?!
+				fd=viddevs[i]->fd_bktr; // FIXME return fd_tuner ?!
+				pthread_mutex_unlock(&vid_mutex);
+				return fd;
 			}
 		}
 
@@ -908,8 +845,8 @@ int vid_start (struct context *cnt)
 
 		fd_bktr=open(conf->video_device, O_RDWR);
 		if (fd_bktr <0) {
-			printf("Failed to open video device %s\n", conf->video_device);
-			syslog(LOG_ERR, "failed to open video device %s: %m", conf->video_device);
+			printf("[%d] Failed to open video device %s\n", cnt->threadnr, conf->video_device);
+			syslog(LOG_ERR, "[%d] Failed to open video device %s: %m", cnt->threadnr, conf->video_device);
 			exit(1);
 		}
 
@@ -917,8 +854,8 @@ int vid_start (struct context *cnt)
 		if (( frequency > 0 ) && ( input = IN_TV )){
 			fd_tuner=open(conf->tuner_device, O_RDWR);
         		if (fd_bktr <0) {
-                		printf("Failed to open video device %s\n", conf->tuner_device);
-                		syslog(LOG_ERR, "failed to open video device %s: %m", conf->tuner_device);
+                		printf("[%d] Failed to open video device %s\n", cnt->threadnr, conf->tuner_device);
+                		syslog(LOG_ERR, "[%d] Failed to open video device %s: %m", cnt->threadnr, conf->tuner_device);
                 		exit(1);
         		}
 		}
@@ -939,7 +876,7 @@ int vid_start (struct context *cnt)
 		viddevs[i]->v4l_curbuffer=0;
 		viddevs[i]->v4l_maxbuffer=1;
 
-		if (!v4l_start (viddevs[i], width, height, input, norm, frequency)) {
+		if (!v4l_start (viddevs[i], width, height, input, norm, frequency, cnt)) {
 			pthread_mutex_unlock(&vid_mutex);
 			return -1;
 		}
@@ -956,7 +893,7 @@ int vid_start (struct context *cnt)
 			case VIDEO_PALETTE_YUV422:
 				cnt->imgs.type=VIDEO_PALETTE_YUV420P;
 			case VIDEO_PALETTE_YUV420P:
-				printf("VIDEO_PALETTE_YUV420P imgs.type\n");
+				printf("[%d] VIDEO_PALETTE_YUV420P imgs.type\n", cnt->threadnr);
 				cnt->imgs.size=(width*height*3)/2;
 				cnt->imgs.motionsize=width*height;
 			break;
@@ -985,57 +922,44 @@ char *vid_next (struct context *cnt, char *map)
 	}
 
 #ifndef WITHOUT_V4L
-	{
-		int i=-1;
-		int width, height;
-		int dev_bktr = cnt->video_dev;
-
-		/* NOTE: Since this is a capture, we need to use capture dimensions. */
-		width = cnt->rotate_data.cap_width;
-		height = cnt->rotate_data.cap_height;
-		
-		while (viddevs[++i])
-			if (viddevs[i]->fd_bktr==dev_bktr)
-				break;
-
-		if (!viddevs[i])
-			return NULL;
-
-		if (viddevs[i]->owner!=cnt->threadnr) {
-			pthread_mutex_lock(&viddevs[i]->mutex);
-			viddevs[i]->owner=cnt->threadnr;
-			viddevs[i]->frames=conf->roundrobin_frames;
-			//viddevs[i]->frames=1;
-			cnt->switched=1;
-		}
-
-
-		v4l_set_input (viddevs[i], map, width, height, conf->input, conf->norm, conf->roundrobin_skip, conf->frequency);
-		ret=v4l_next (viddevs[i], map, width, height);
-
-		if (--viddevs[i]->frames <= 0) {
-			viddevs[i]->owner=-1;
-			pthread_mutex_unlock(&viddevs[i]->mutex);
-		}
 	
- 		if(cnt->rotate_data.degrees > 0) {
-			/* rotate the image as specified */
-			rotate_map(map, cnt);
- 		}
+	int i=-1;
+	int width, height;
+	int dev_bktr = cnt->video_dev;
+
+	/* NOTE: Since this is a capture, we need to use capture dimensions. */
+	width = cnt->rotate_data.cap_width;
+	height = cnt->rotate_data.cap_height;
+		
+	while (viddevs[++i])
+		if (viddevs[i]->fd_bktr==dev_bktr)
+			break;
+
+	if (!viddevs[i])
+		return NULL;
+
+	if (viddevs[i]->owner!=cnt->threadnr) {
+		pthread_mutex_lock(&viddevs[i]->mutex);
+		viddevs[i]->owner=cnt->threadnr;
+		viddevs[i]->frames=conf->roundrobin_frames;
+		//viddevs[i]->frames=1;
+		cnt->switched=1;
 	}
+
+
+	v4l_set_input (viddevs[i], map, width, height, conf->input, conf->norm, conf->roundrobin_skip, conf->frequency, cnt);
+	ret=v4l_next (viddevs[i], map, width, height, cnt);
+
+	if (--viddevs[i]->frames <= 0) {
+		viddevs[i]->owner=-1;
+		pthread_mutex_unlock(&viddevs[i]->mutex);
+	}
+	
+ 	if(cnt->rotate_data.degrees > 0) {
+		/* rotate the image as specified */
+		rotate_map(map, cnt);
+ 	}
+	
 #endif /*WITHOUT_V4L*/
 	return ret;
 }
-
-#ifndef WITHOUT_V4L
-int vid_startpipe (char *devname, int width, int height, int type)
-{
-	return v4l_startpipe (devname, width, height, type);
-}
-
-int vid_putpipe (int dev, char *image, int size)
-{
-	return v4l_putpipe (dev, image, size);
-}
-
-#endif /*WITHOUT_V4L*/
