@@ -85,8 +85,6 @@ char *vid_next (struct context *, char *map);
 
 #ifndef WITHOUT_V4L
 void vid_init(void);
-int vid_startpipe (char *devname_bktr,/*char *devname_tuner,*/ int width, int height, int);
-int vid_putpipe (int dev, char *image, int);
 void vid_close(void);
 void vid_cleanup(void);
 #endif
